from __future__ import annotations

from typing import Any, NamedTuple, Type, TypeVar

OptionT = TypeVar("OptionT", bound="Option")


class Option(NamedTuple):
    """Base class for options. All subclasses should inherit from this class and Enum."""

    description: str
    data: Any

    def __str__(self):
        return self.description

    @classmethod
    def find_by_description(cls: Type[OptionT], description: str) -> OptionT:
        """Get an option from a description

        Examples:

        - ``Variant.find_by_description("Antichess")``
        - ``Color.find_by_description("White")``
        - ``TimeMode.find_by_description("Real-time")``

        Parameters
        ----------
        description: :class:`str`
            The description of the option

        Returns
        -------
        :class:`Option`
            The option with the description

        Raises
        ------
        ValueError
            If the description is not found
        """
        for option in cls:  # type: ignore  # must be an Enum
            if description == option.description:
                return option
        raise ValueError("Unknown option")

    @classmethod
    def find_by_data(cls: Type[OptionT], data: str) -> OptionT:
        """Get an option from its data

        Examples:

        - ``Variant.find_by_data("antichess")``
        - ``Color.find_by_data("white")``
        - ``TimeMode.find_by_data("ultraBullet")``

        Parameters
        ----------
        data: :class:`str`
            The data of the option

        Returns
        -------
        :class:`Option`
            The option with the data

        Raises
        ------
        ValueError
            If the option doesn't exist
        """
        for option in cls:  # type: ignore  # must be an Enum
            if data == option.data:
                return option
        raise ValueError("Unknown option")

    @classmethod
    def find(cls: Type[OptionT], key: str) -> OptionT:
        """Get an option from description or data

        Examples:

        - ``Variant.find("Antichess")``
        - ``Variant.find("ultraBullet")``
        - ``TimeMode.find("Real-time")``
        - ``TimeMode.find("realTime")``

        Parameters
        ----------
        key: :class:`str`
            The description or data of the option

        Returns
        -------
        :class:`Option`
            The option with the description or data

        Raises
        ------
        ValueError
            If the option doesn't exist
        """
        try:
            return cls.find_by_description(key)
        except ValueError:
            return cls.find_by_data(key)



from .exceptions import BadArgumentError, BaseError, HttpError
from .match import CorrespondenceMatch, Match, MatchInfo, RealTimeMatch, UnlimitedMatch
from .option import Option
from .types import Color, TimeControl, TimeControlType, TimeMode, User, Variant

__version__ = "1.1.1"

__all__ = [
    "__version__",
    "BaseError",
    "HttpError",
    "BadArgumentError",
    "MatchInfo",
    "Match",
    "RealTimeMatch",
    "CorrespondenceMatch",
    "UnlimitedMatch",
    "Option",
    "Variant",
    "TimeMode",
    "Color",
    "TimeControlType",
    "TimeControl",
    "User",
]



# /usr/bin/env python
import os
import re
from setuptools import setup

version = ""
with open(os.path.join("play_lichess", "__init__.py"), "r", encoding="utf-8") as f:
    version_match = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE
    )
    version = version_match.group(1) if version_match else ""

if not version:
    raise RuntimeError("version is not set")

long_description = ""
if os.path.exists("README.md"):
    with open("README.md", "r", encoding="utf-8") as f:
        long_description = f.read()

requirements = []
if os.path.exists("requirements.txt"):
    with open("requirements.txt", "r", encoding="utf-8") as f:
        requirements = f.read().splitlines()

setup(
    name="play-lichess",
    version=version,
    author="Jonah Lawrence",
    author_email="jonah@freshidea.com",
    description="Module for creating match links on Lichess that players can join",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DenverCoder1/play-lichess",
    project_urls={
        "Bug Tracker": "https://github.com/DenverCoder1/play-lichess/issues",
    },
    packages=["play_lichess"],
    package_data={"play_lichess": ["py.typed"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Utilities",
    ],
    python_requires=">=3.8",
    install_requires=[requirements],
)
